#-----------------------------------------------------------------||||||||||||--
# Name:          fontLibrary.py
# Purpose:       contains bit-mapped fonts and tools to access their data.
#
# Authors:       Christopher Ariza
#
# Copyright:     (c) 2004-2010 Christopher Ariza
# License:       GPL
#-----------------------------------------------------------------||||||||||||--


import string
import unittest, doctest

from athenaCL.libATH import typeset
_MOD = 'fontLibrary.py'


#-----------------------------------------------------------------||||||||||||--
# examples from: http://www.dsg4.com/04/extra/bitmap/index.html


# calculate wdith algorithmically
fontMicro = {    # a-o
                     "a" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (1,1,1,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (0,0,0,0,0),),
                     "b" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (1,0,0,1,0),
                     (1,1,1,1,0),
                     (1,0,0,1,0),
                     (1,1,1,0,0),
                     (0,0,0,0,0),),
                     "c" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (1,0,0,0,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "d" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,1,1,0,0),
                     (0,0,0,0,0),),

                     "e" : (
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (1,0,0,0,0),
                     (1,1,1,1,0),
                     (1,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),),
                     "f" : (
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (1,0,0,0,0),
                     (1,1,1,1,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "g" : (
                     (0,0,0,0,0),
                     (0,1,1,1,0),
                     (1,0,0,0,0),
                     (1,0,1,1,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "h" : (
                     (0,0,0,0,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,1,1,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (0,0,0,0,0),),
                     "i" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "j" : (
                     (0,0,0,0,0),
                     (0,0,0,1,0),
                     (0,0,0,1,0),
                     (0,0,0,1,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "k" : (
                     (0,0,0,0,0),
                     (1,0,0,1,0),
                     (1,0,1,0,0),
                     (1,1,0,0,0),
                     (1,0,1,0,0),
                     (1,0,0,1,0),
                     (0,0,0,0,0),),
                     "l" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),),
                     "m" : (
                     (0,0,0,0,0),
                     (1,0,0,0,1),
                     (1,1,0,1,1),
                     (1,0,1,0,1),
                     (1,0,0,0,1),
                     (1,0,0,0,1),
                     (0,0,0,0,0),),
                     "n" : (
                     (0,0,0,0,0),
                     (1,0,0,0,1),
                     (1,1,0,0,1),
                     (1,0,1,0,1),
                     (1,0,0,1,1),
                     (1,0,0,0,1),
                     (0,0,0,0,0),),
                     "o" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     # p-z
                     "p" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (1,0,0,1,0),
                     (1,1,1,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "q" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,1,0),),
                     "r" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (1,0,0,1,0),
                     (1,1,1,0,0),
                     (1,0,1,0,0),
                     (1,0,0,1,0),
                     (0,0,0,0,0),),
                     "s" : (
                     (0,0,0,0,0),
                     (0,1,1,1,0),
                     (1,0,0,0,0),
                     (0,1,1,0,0),
                     (0,0,0,1,0),
                     (1,1,1,0,0),
                     (0,0,0,0,0),),
                     "t" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,0,0,0,0),),
                     "u" : (
                     (0,0,0,0,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "v" : (
                     (0,0,0,0,0),
                     (1,0,0,0,1),
                     (1,0,0,0,1),
                     (0,1,0,1,0),
                     (0,1,0,1,0),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     "w" : (
                     (0,0,0,0,0),
                     (1,0,0,0,1),
                     (1,0,0,0,1),
                     (1,0,1,0,1),
                     (1,0,1,0,1),
                     (0,1,0,1,0),
                     (0,0,0,0,0),),
                     "x" : (
                     (0,0,0,0,0),
                     (1,0,0,0,1),
                     (0,1,0,1,0),
                     (0,0,1,0,0),
                     (0,1,0,1,0),
                     (1,0,0,0,1),
                     (0,0,0,0,0),),
                     "y" : (
                     (0,0,0,0,0),
                     (1,0,0,0,1),
                     (0,1,0,1,0),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     "z" : (
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,1,0),
                     (0,1,1,0,0),
                     (1,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),),

                     # numbers
                     "1" : (
                     (0,0,0,0,0),
                     (1,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (1,1,1,0,0),
                     (0,0,0,0,0),),
                     "2" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (0,0,0,1,0),
                     (0,1,1,0,0),
                     (1,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),),
                     "3" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (0,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,1,0),
                     (1,1,1,0,0),
                     (0,0,0,0,0),),
                     "4" : (
                     (0,0,0,0,0),
                     (1,0,1,0,0),
                     (1,0,1,0,0),
                     (1,1,1,1,0),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     "5" : (
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (1,0,0,0,0),
                     (1,1,1,0,0),
                     (0,0,0,1,0),
                     (1,1,1,0,0),
                     (0,0,0,0,0),),
                     "6" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,0,0),
                     (1,1,1,0,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "7" : (
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,1,0),
                     (0,0,1,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,0,0,0,0),),
                     "8" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "9" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (0,1,1,1,0),
                     (0,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "0" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (1,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     # symbols
                     "!" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "@" : (
                     (0,0,0,0,0),
                     (0,1,1,1,0),
                     (1,0,1,0,1),
                     (1,0,1,1,0),
                     (1,0,0,0,0),
                     (0,1,1,1,0),
                     (0,0,0,0,0),),
                     "#" : (
                     (0,0,0,0,0),
                     (0,1,0,1,0),
                     (1,1,1,1,1),
                     (0,1,0,1,0),
                     (1,1,1,1,1),
                     (0,1,0,1,0),
                     (0,0,0,0,0),),
                     "$" : (
                     (0,0,1,0,0),
                     (0,1,0,1,0),
                     (1,0,0,0,0),
                     (0,1,1,0,0),
                     (0,0,0,1,0),
                     (1,1,1,0,0),
                     (0,0,1,0,0),),
                     "%" : (
                     (0,0,0,0,0),
                     (1,1,0,1,0),
                     (1,1,0,1,0),
                     (0,0,1,0,0),
                     (0,1,0,1,1),
                     (0,1,0,1,1),
                     (0,0,0,0,0),),
                     "^" : (
                     (0,1,0,0,0),
                     (1,0,1,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "&" : (
                     (0,0,1,0,0),
                     (0,1,1,1,0),
                     (1,0,0,0,0),
                     (0,1,1,0,0),
                     (1,0,0,0,0),
                     (0,1,1,1,0),
                     (0,0,1,0,0),),
                     "*" : (
                     (0,0,0,0,0),
                     (0,0,1,0,0),
                     (1,0,1,0,1),
                     (0,1,1,1,0),
                     (1,0,1,0,1),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     "(" : (
                     (0,0,0,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,0,0,0),),
                     ")" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "_" : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),),
                     "-" : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "+" : (
                     (0,0,0,0,0),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (1,1,1,1,1),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     "=" : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),
                     (1,1,1,1,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "`" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "~" : (
                     (0,0,0,0,0),
                     (0,1,0,1,0),
                     (1,0,1,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "[" : (
                     (0,0,0,0,0),
                     (1,1,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,1,0,0,0),
                     (0,0,0,0,0),),
                     "]" : (
                     (0,0,0,0,0),
                     (1,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (0,1,0,0,0),
                     (1,1,0,0,0),
                     (0,0,0,0,0),),

                     "\\" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     "{" : (
                     (0,0,0,0,0),
                     (0,1,1,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),),
                     "}" : (
                     (0,0,0,0,0),
                     (1,1,0,0,0),
                     (0,1,0,0,0),
                     (0,0,1,0,0),
                     (0,1,0,0,0),
                     (1,1,0,0,0),
                     (0,0,0,0,0),),
                     "|" : (
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),),
                     ";" : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,0,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     ":" : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "'" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "\"" : (
                     (0,0,0,0,0),
                     (1,0,1,0,0),
                     (1,0,1,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),),
                     "," : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),),
                     "." : (
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "/" : (
                     (0,0,0,0,0),
                     (0,0,1,0,0),
                     (0,0,1,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "<" : (
                     (0,0,0,0,0),
                     (0,0,1,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,1,0,0),
                     (0,0,0,0,0),),
                     ">" : (
                     (0,0,0,0,0),
                     (1,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,1,0,0),
                     (0,1,0,0,0),
                     (1,0,0,0,0),
                     (0,0,0,0,0),),
                     "?" : (
                     (0,0,0,0,0),
                     (1,1,1,0,0),
                     (0,0,0,1,0),
                     (0,1,1,0,0),
                     (0,0,0,0,0),
                     (0,1,0,0,0),
                     (0,0,0,0,0),),
        }


#-----------------------------------------------------------------||||||||||||--


fontMacro = {

        "!" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "\"" : (
        (1,1,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "#" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "$" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "%" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "&" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "'" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "(" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        ")" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "*" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "+" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "," : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "." : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "/" : (
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "0" : (
        (0,0,0,1,1,1,1,1,1,1,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,1,1,0,1,1,0,0),
        (1,1,0,0,0,0,1,1,0,0,0,1,1,0,0),
        (1,1,0,0,0,1,1,0,0,0,0,1,1,0,0),
        (1,1,0,1,1,0,0,0,0,0,0,1,1,0,0),
        (0,1,1,0,0,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,1,1,1,1,1,1,1,0,0,0,0,0),),
        "1" : (
        (1,1,1,1,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),),
        "2" : (
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,0,1,1,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),),
        "3" : (
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0,0,0,0,0),),
        "4" : (
        (0,0,0,0,0,0,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,1,1,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,1,1,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),),
        "5" : (
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0,0,0,0,0),),
        "6" : (
        (0,0,0,0,1,1,1,1,1,1,1,1,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,1,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (0,1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (0,0,0,1,1,1,1,1,1,1,1,0,0,0,0),),
        "7" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),),
        "8" : (
        (0,0,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,1,1,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,1,1,1,0,0,0,0,0,0),),
        "9" : (
        (0,0,1,1,1,1,1,1,1,1,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,1,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,1,0,0,0,0,0,0,0,1,1,1,0,0),
        (0,0,1,1,1,1,1,1,1,1,0,1,1,0,0),
        (0,0,0,0,0,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (0,1,1,1,1,1,1,1,1,0,0,0,0,0,0),),
        ":" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        ";" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "<" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        ">" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "?" : (
        (0,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,1,1,1,1,0,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),),
        "@" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "[" : (
        (1,1,1,1,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0,0,0,0,0),),
        "\\" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,0,0,0,0,0,0),),
        "]" : (
        (1,1,1,1,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0,0,0,0,0),),
        "^" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "-" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),      
        "_" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),),
        "`" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "{" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "|" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "}" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "~" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0),),

        "a" : (
        (0,0,0,0,0,0,1,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,1,1,0,1,1,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,1,1,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,1,1,0,0,0),
        (0,0,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (0,1,1,0,0,0,0,0,0,0,0,0,1,1,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,1,1),),
        "b" : (
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),),
        "c" : (
        (0,0,0,1,1,1,1,1,1,1,1,1,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,1,1,1,1,1,1,1,0,0,0),),
        "d" : (
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,1,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),),
        "e" : (
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),),
        "f" : (
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "g" : (
        (0,0,0,1,1,1,1,1,1,1,1,1,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,1,1,1,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,1,1,1,1,1,1,1,1,0,0,0,0),),
        "h" : (
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),),
        "i" : (
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),),
        "j" : (
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0,0,0,0,0),),
        "k" : (
        (1,1,0,0,0,0,0,0,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,1,1,1,1,0,0,0,0,0),
        (1,1,0,1,1,1,1,1,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,1,1,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,1,1,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,1,0,0),),
        "l" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),),
        "m" : (
        (1,1,1,1,0,0,0,0,0,1,1,1,1,0,0),
        (1,1,1,1,1,0,0,0,1,1,1,1,1,0,0),
        (1,1,0,1,1,1,0,1,1,1,0,1,1,0,0),
        (1,1,0,0,1,1,1,1,1,0,0,1,1,0,0),
        (1,1,0,0,0,1,1,1,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),),
        "n" : (
        (1,1,1,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,1,1,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,1,1,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,1,1,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,1,1,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,1,1,1,0,0,0,0),),
        "o" : (
        (0,0,0,1,1,1,1,1,1,1,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (0,1,1,0,0,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,1,1,1,1,1,1,1,0,0,0,0,0),),
        "p" : (
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "q" : (
        (0,0,0,1,1,1,1,1,1,1,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,1,1,1,0,1,1,0,0),
        (0,1,1,0,0,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,1,1,1,1,1,1,1,0,1,1,1,0),),
        "r" : (
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,1,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,1,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,1,0,0),),
        "s" : (
        (0,0,1,1,1,1,1,1,1,1,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,1,1,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0,0,0,0,0,0),),
        "t" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0,0),),
        "u" : (
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,1,1,1,0,0,0,0,0,0),),
        "v" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,1,1,0,0,0,0,0,0,0,0,0,1,1,0),
        (0,0,1,1,0,0,0,0,0,0,0,1,1,0,0),
        (0,0,0,1,1,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,1,1,0,0,0),
        (0,0,0,0,1,1,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,1,0,0,0,0,0,0),),
        "w" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,0,0,0,0,0,0,0,0,0,1,1,1),
        (0,1,1,0,0,0,0,1,0,0,0,0,1,1,0),
        (0,1,1,0,0,0,1,1,1,0,0,0,1,1,0),
        (0,1,1,0,0,1,1,0,1,1,0,0,1,1,0),
        (0,1,1,0,1,1,0,0,0,1,1,1,1,0,0),
        (0,0,1,1,1,0,0,0,0,0,1,1,1,0,0),),
        "x" : (
        (1,1,1,0,0,0,0,0,0,0,1,1,1,0,0),
        (0,1,1,1,0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,1,1,1,0,1,1,1,0,0,0,0,0),
        (0,0,0,0,1,1,1,1,1,0,0,0,0,0,0),
        (0,0,0,0,1,1,1,1,1,0,0,0,0,0,0),
        (0,0,0,1,1,1,0,1,1,1,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,1,1,1,0,0),),
        "y" : (
        (1,1,0,0,0,0,0,0,1,1,0,0,0,0,0),
        (0,1,1,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,1,1,0,0,0,0,0,0,0),
        (0,0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0,0),),
        "z" : (
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,0,0,0,0),),

}


fontPoster = {
        "a" : (
        (0,0,1,1,1,1,0,0,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),),
        "b" : (
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),),
        "c" : (
        (0,0,1,1,1,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (0,1,1,1,1,1,1,1,0,0,0),
        (0,0,1,1,1,1,1,1,0,0,0),),
        "d" : (
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),),
        "e" : (
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),),
        "f" : (
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),
        "g" : (
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,1,1,1,1,0,0,0),
        (1,1,1,0,1,1,1,1,0,0,0),
        (1,1,1,0,0,0,1,1,0,0,0),
        (1,1,1,0,0,0,1,1,0,0,0),
        (0,1,1,1,1,1,1,1,0,0,0),
        (0,0,1,1,1,0,1,1,0,0,0),),
        "h" : (
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),),
        "i" : (
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),
        "j" : (
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (1,1,0,0,0,1,1,1,0,0,0),
        (1,1,0,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),),
        "k" : (
        (1,1,1,0,0,0,1,1,1,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,0,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,0,1,1,1,0,0),),
        "l" : (
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),),
        "m" : (
        (1,1,1,0,0,0,0,0,1,1,1),
        (1,1,1,1,0,0,0,1,1,1,1),
        (1,1,1,1,1,0,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,0,1,1,1,0,1,1,1),
        (1,1,1,0,0,1,0,0,1,1,1),
        (1,1,1,0,0,0,0,0,1,1,1),
        (1,1,1,0,0,0,0,0,1,1,1),
        (1,1,1,0,0,0,0,0,1,1,1),
        (1,1,1,0,0,0,0,0,1,1,1),),
        "n" : (
        (1,1,1,0,0,0,0,0,0,1,1,1),
        (1,1,1,1,0,0,0,0,0,1,1,1),
        (1,1,1,1,1,0,0,0,0,1,1,1),
        (1,1,1,1,1,1,0,0,0,1,1,1),
        (1,1,1,0,1,1,1,0,0,1,1,1),
        (1,1,1,0,0,1,1,1,0,1,1,1),
        (1,1,1,0,0,0,1,1,1,1,1,1),
        (1,1,1,0,0,0,0,1,1,1,1,1),
        (1,1,1,0,0,0,0,0,1,1,1,1),
        (1,1,1,0,0,0,0,0,0,1,1,1),),
        "o" : (
        (0,0,1,1,1,1,0,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),),
        "p" : (
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),
        "q" : (
        (0,0,1,1,1,1,0,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),),
        "r" : (
        (1,1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,0,1,1,1,0,0,0,0),
        (1,1,1,0,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),),
        "s" : (
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (0,1,1,1,1,1,0,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),),
        "t" : (
        (1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),),
        "u" : (
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),),
        "v" : (
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,0,0,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0,0),),
        "w" : (
        (1,1,1,0,1,1,0,1,1,1,0),
        (1,1,1,0,1,1,0,1,1,1,0),
        (1,1,1,0,1,1,0,1,1,1,0),
        (1,1,1,0,1,1,0,1,1,1,0),
        (1,1,1,0,1,1,0,1,1,1,0),
        (1,1,1,0,1,1,0,1,1,1,0),
        (1,1,1,0,1,1,0,1,1,1,0),
        (0,1,1,1,1,1,1,1,1,0,0),
        (0,0,1,1,1,1,1,1,0,0,0),
        (0,0,0,1,0,0,1,0,0,0,0),),
        "x" : (
        (1,1,1,0,0,0,1,1,1,0,0),
        (1,1,1,0,0,0,1,1,1,0,0),
        (0,1,1,1,0,1,1,1,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,1,1,1,0,1,1,1,0,0,0),
        (1,1,1,0,0,0,1,1,1,0,0),
        (1,1,1,0,0,0,1,1,1,0,0),),
        "y" : (
        (1,1,1,0,0,0,0,0,1,1,1),
        (0,1,1,1,0,0,0,1,1,1,0),
        (0,0,1,1,1,0,1,1,1,0,0),
        (0,0,0,1,1,1,1,1,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),),
        "z" : (
        (1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,0,0,0,1,1,1,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),),

        "0" : (
        (0,0,1,1,1,1,0,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),),

        "1" : (
        (0,0,1,0,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),
        "2" : (
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),),
        "3" : (
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),),
        "4" : ( 
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),),
        "5" : (
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),),
        "6" : (
        (0,0,1,1,1,1,1,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),),
        "7" : (
        (1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,0,0,0,1,1,1,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),
        "8" : (
        (0,0,1,1,1,1,0,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0),),
        "9" : (
        (0,0,1,1,1,1,0,0,0,0,0),
        (0,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,1,1,1,1,1,1,1,0,0,0),
        (0,0,1,1,1,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,0,0,0,0,0),),



        "'" : (
        (1,1,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0),
        (0,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),),
        '"' : (
        (1,1,0,1,1,0,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0,0),
        (0,1,0,0,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),),

        "?" : (
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0,0),
        (1,1,1,0,0,1,1,1,0,0,0),
        (0,0,0,0,1,1,1,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0,0),),
        "i" : (
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),
        "." : (
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0,0),),


}


fontCapital = {
        "a" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "b" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "c" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "d" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "e" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "f" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "g" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "h" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "i" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "j" : (
        (0,0,0,0,0,0,0,0,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "k" : (
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "l" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "m" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "n" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "o" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "p" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "q" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),),
        "r" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "s" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "t" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "u" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "v" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "w" : (
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "x" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,1,1,1,1,1,1,1,1,1,1,0,0),
        (0,0,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "y" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "z" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "0" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "1" : (
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "2" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "3" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "4" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "5" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "6" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "7" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "8" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "9" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "." : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "," : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),),
        "-" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "+" : (
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "=" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "!" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "?" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "@" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "$" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,1,1,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,1,1,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "&" : (
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "'" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        '"' : (
        (1,1,0,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "{" : (
        (0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "}" : (
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,1,1,1,1,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "]" : (
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "[" : (
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        ">" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "<" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        ")" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "(" : (
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),

        ":" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        ";" : (
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "/" : (
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "\\" : (
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,1,1,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "%" : (
        (1,1,1,1,1,1,1,1,1,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),
        "#" : (
        (0,0,1,1,0,0,0,0,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,1,1,0,0,0,0,0,0,1,1,0,0),
        (0,0,1,1,0,0,0,0,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1,1,1,1,1),
        (0,0,1,1,0,0,0,0,0,0,1,1,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0,0,0,0,0),),

}



fontStrong = {
        "a" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "b" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),),
        "c" : (
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),),
        "d" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),),
        "e" : (
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),),
        "f" : (
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "g" : (
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,1,1,1,1,1,1),
        (1,1,0,0,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),),
        "h" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "i" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "j" : (
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "k" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,1,1,1),
        (1,1,0,0,0,0,1,1,1,0),
        (1,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,1,1,1,0),
        (1,1,0,0,0,0,0,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "l" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),),
        "m" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,0,0,0,0,1,1,1),
        (1,1,1,1,0,0,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,1,1,1,1,0,1,1),
        (1,1,0,0,1,1,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "n" : (
        (1,1,1,0,0,0,0,0,1,1),
        (1,1,1,1,0,0,0,0,1,1),
        (1,1,1,1,1,0,0,0,1,1),
        (1,1,0,1,1,1,0,0,1,1),
        (1,1,0,0,1,1,1,0,1,1),
        (1,1,0,0,0,1,1,1,1,1),
        (1,1,0,0,0,0,1,1,1,1),
        (1,1,0,0,0,0,0,1,1,1),),
        "o" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "p" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "q" : (
        (0,1,1,1,1,1,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,0,0,0,0,0,1,1,0),
        (1,1,0,0,0,0,0,1,1,0),
        (1,1,0,0,0,0,0,1,1,0),
        (1,1,0,0,0,0,0,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),),
        "r" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,0,0,0,0,0,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "s" : (
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0),
        (0,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),),
        "t" : (
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),),
        "u" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "v" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,0,0,0,0,1,1,1),
        (0,1,1,1,0,0,1,1,1,0),
        (0,0,1,1,1,1,1,1,0,0),
        (0,0,0,1,1,1,1,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),),
        "w" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,1,1,0,0,1,1),
        (1,1,0,1,1,1,1,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,0,0,1,1,1,1),
        (1,1,1,0,0,0,0,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "x" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,0,0,0,0,1,1,1),
        (0,1,1,1,0,0,1,1,1,0),
        (0,0,1,1,1,1,1,1,0,0),
        (0,0,1,1,1,1,1,1,0,0),
        (0,1,1,1,0,0,1,1,1,0),
        (1,1,1,0,0,0,0,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),),
        "y" : (
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "z" : (
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),),

        "0" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,0,1,1,0,0,1,1),
        (1,1,0,0,1,1,0,0,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "1" : (
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),),
        "2" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),),
        "3" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),),
        "4" : (
        (1,1,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,1,1,0,0),
        (1,1,0,0,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,1,1,0,0),
        (0,0,0,0,0,0,1,1,0,0),),
        "5" : (
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,0),),
        "6" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "7" : (
        (1,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,0,0,0,0,0,0,0,1,1),),
        "8" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "9" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "," : (
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "." : (
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        ";" : (
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        ":" : (
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),),
        "!" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "?" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (0,0,0,0,1,1,1,1,1,1),
        (0,0,0,0,1,1,1,1,1,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),),
        "+" : (
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),
        (0,0,0,0,1,1,0,0,0,0),),
        "-" : (
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),),
        "@" : (
        (0,1,1,1,1,1,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,0,0,1,1),
        (1,1,0,1,1,1,1,1,1,1),
        (1,1,0,1,1,1,1,1,1,1),
        (1,1,0,1,1,0,0,0,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,1,1,0),),
        "%" : (
        (1,1,1,0,0,0,0,1,1,0),
        (1,1,1,0,0,0,1,1,1,0),
        (1,1,1,0,0,1,1,1,0,0),
        (0,0,0,0,1,1,1,0,0,0),
        (0,0,0,1,1,1,0,0,0,0),
        (0,0,1,1,1,0,0,1,1,1),
        (0,1,1,1,0,0,0,1,1,1),
        (0,1,1,0,0,0,0,1,1,1),),
        "&" : (
        (0,1,1,1,1,1,1,0,0,0),
        (1,1,1,1,1,1,1,1,0,0),
        (1,1,0,0,0,0,1,1,0,0),
        (0,1,1,1,1,1,1,0,1,1),
        (0,1,1,1,1,1,1,1,1,1),
        (1,1,0,0,0,0,1,1,1,0),
        (1,1,1,1,1,1,1,1,1,1),
        (0,1,1,1,1,1,1,0,1,1),),
        "(" : (
        (0,0,1,1,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),),
        ")" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "[" : (
        (1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),),
        "]" : (
        (1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),
        (1,1,1,1,0,0,0,0,0,0),),
        "{" : (
        (0,0,1,1,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,0,0,0,0,0,0),),
        "}" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,1,1,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "<" : (
        (0,0,0,1,1,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,0,0,0,0,0),),
        ">" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "'" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),),
        '"' : (
        (1,1,0,1,1,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0),
        (1,1,0,1,1,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),),
        "/" : (
        (0,0,0,0,0,0,1,1,0,0),
        (0,0,0,0,0,1,1,1,0,0),
        (0,0,0,0,1,1,1,0,0,0),
        (0,0,0,1,1,1,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (1,1,0,0,0,0,0,0,0,0),),
        "\\" : (
        (1,1,0,0,0,0,0,0,0,0),
        (1,1,1,0,0,0,0,0,0,0),
        (0,1,1,1,0,0,0,0,0,0),
        (0,0,1,1,1,0,0,0,0,0),
        (0,0,0,1,1,1,0,0,0,0),
        (0,0,0,0,1,1,1,0,0,0),
        (0,0,0,0,0,1,1,1,0,0),
        (0,0,0,0,0,0,1,1,0,0),),
        "=" : (
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0),
        (0,0,0,0,0,0,0,0,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,0,0,0,0,0,0,0,0),),
        "#" : (
        (0,0,1,1,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,1,1,0,0,1,1,0,0),
        (0,0,1,1,0,0,1,1,0,0),
        (1,1,1,1,1,1,1,1,1,1),
        (1,1,1,1,1,1,1,1,1,1),
        (0,0,1,1,0,0,1,1,0,0),),


}




def genCharTemplate(count=2, w=15, h=8):
    chars = fontMicro.keys()
    chars.sort()
    print chars
    msg = []
    for name in chars:
        msg.append('\t\t"%s" : (\n' % name)
        for x in range(0, h):
            newRow = [0,] * w
            newRow = tuple(newRow)
            msg.append('\t\t%s,' % str(newRow).replace(' ',''))
            if x != h - 1: # if not last
                msg.append('\n')
            else:
                msg.append('),\n')
    print ''.join(msg)



#-----------------------------------------------------------------||||||||||||--

class FontBitMap:
    def __init__(self, msg, font='micro', kernEqual=0, kernBit=None,
                 vertBit=None, charWrapWidth= None):
        """kern equal determins if char width is equal, or scaled
        kernBit is total width of each char in bits
        vertBit is total height of each line bit
        charWidth will wrap lines
        """
        if charWrapWidth == None: # all on one line
            self.msgLines = [msg,]
        else: # wrap lines
            msg = typeset.wrapText(msg, charWrapWidth)
            self.msgLines = msg.split('\n')

        if font == 'micro':
            self.font = fontMicro
        elif font == 'macro':
            self.font = fontMacro
        elif font == 'poster':
            self.font = fontPoster
        elif font == 'capital':
            self.font = fontCapital
        elif font == 'strong':
            self.font = fontStrong
        else:
            raise ValueError, 'no such font %r' % font

        self.fontChar = fontMicro.keys()
        self.wFont = len(self.font['a'][0])
        self.hFont = len(self.font['a'])

        self.kernEqual = kernEqual
        self.kernBit = kernBit
        self.vertBit = vertBit

    def _findCharWidth(self, char): 
        """find max width for a given char, from left"""
        charWidth = 0
        for line in char:
            line = list(line)
            q = 0
            line.reverse()
            for bit in line: # go from end, find first 1
                if bit != 0: 
                    break
                q = q + 1
            p = len(line) - q # subtract left extra space
            if p >= charWidth:
                charWidth = p
        return charWidth

    def _shaveChar(self, char):
        """remove any vertical white space from char
        always from right: assume all chars are left flushed"""
        charWidth = self._findCharWidth(char)
        newChar = []
        for line in char:
            line = list(line)
            newChar.append(line[0:charWidth])
        return newChar

    def _padChar(self, char, padSide, padBit):
        """add vertical white space to char, on either l or r side"""
        newChar = []
        for line in char:
            line = list(line)
            if padSide.lower() == 'l':
                newChar.append(([0]*padBit) + line)
            elif padSide.lower() == 'r':
                newChar.append(line + ([0]*padBit))
        return newChar

    def _balanceChar(self, char, bitWidth):
        """add equal white space to each side of char"""
        bitTotal = bitWidth - self._findCharWidth(char)
        if bitTotal < 0:
            raise ValueError, 'bitWidth is too small'
        if bitTotal == 0: # already is desired width
            return char
        if bitTotal % 2 == 0: # its even
            bitL = bitTotal / 2
            bitR = bitTotal / 2
        else: # odd
            bitL = bitTotal / 2
            bitR = (bitTotal / 2) + 1 # add extra value
        char = self._padChar(char, 'l', bitL)
        char = self._padChar(char, 'r', bitR)
        return char

    def _kernChar(self, char):
        char = self._shaveChar(char)
        if self.kernEqual: # check if char should be equalled
            char = self._balanceChar(self, char, self.wFont)
        if self.kernBit == None:
            return char
        else: # add space after char, abs
            char = self._padChar(char, 'r', self.kernBit)
            return char

    def _getSpaceChar(self):
        w = self.wFont
        w = w + self.kernBit # add kerning
        char = []
        for row in range(0, self.hFont):
            char.append([0,]*w)
        return char

    def getChar(self, char):
        char = char.lower()
        if char in self.font.keys():
            return self._kernChar(self.font[char])
        elif char == ' ': # get space
            return self._getSpaceChar()
        else: # ? for unknown chars
            return self.font['?']

    def _sumHorizontal(self, gridList):
        """take a list of grids and make them into one grid
        creates a line of text
        auto trims left white space from line"""
        sum = []
        rowCount = 0
        for grid in gridList: # get no of rows
            if len(grid) >= rowCount:
                rowCount = len(grid)
        for row in range(0, rowCount):
            sum.append([])
        for grid in gridList:
            for row in range(0, rowCount):
                sum[row] = sum[row] + list(grid[row])
        sum = self._shaveChar(sum)
        return sum
    
    def _sumVertical(self, bitLines, align='l'):
        """give a lost of grids, sum vertically"""
        if self.vertBit == None: # do nothing
            vPad = 0
        else:
            vPad = self.vertBit
        # get max bits used in all lines
        bitWidth = 0
        for rowGrid in bitLines: # bundle
            x = self._findCharWidth(rowGrid)
            if x >= bitWidth:
                bitWidth = x

        masterGrid = []
        for rowGrid in bitLines:
            if rowGrid == []:
                print _MOD, 'error'
                continue
            if len(rowGrid[0]) != bitWidth:
                bitDif = bitWidth - len(rowGrid[0])
                if align == 'l': # left align means right pad
                    rowGrid = self._padChar(rowGrid, 'r', bitDif)
                else:
                    rowGrid = self._padChar(rowGrid, 'l', bitDif)   
            for row in rowGrid:
                masterGrid.append(row)
            for i in range(0, vPad):
                pad = [0] * len(rowGrid[0])
                # was, incorrectly, as below
                # pad = [0] * len(lineGrid[0])
                masterGrid.append(pad)
        return masterGrid

    def encode(self):
        bitLines = []
        for lineNo in range(0,len(self.msgLines)):
            bitLines.append([])
        # do each line
        for lineNo in range(0,len(self.msgLines)):
            bitRow = bitLines[lineNo]
            msgRow = self.msgLines[lineNo]
            #print msgRow
            newBitRow = []
            for char in msgRow:
                bitChar = self.getChar(char)
                newBitRow.append(bitChar)
            bitLines[lineNo] = self._sumHorizontal(newBitRow)
            #self._printChar(bitLines[lineNo])
        bitLines = self._sumVertical(bitLines)
        #self._printChar(bitLines)
        return bitLines

    def _printChar(self, char):
        for row in char:
            rowStr = str(row)
            rowStr = rowStr.replace(',','')
            rowStr = rowStr.replace(' ','')
            rowStr = rowStr.replace('1','.')
            rowStr = rowStr.replace('0',' ')
            print rowStr



#-----------------------------------------------------------------||||||||||||--
class Test(unittest.TestCase):
    
    def runTest(self):
        pass
            
    def testDummy(self):
        self.assertEqual(True, True)

    def testBasic(self):
        for font in ['micro', 'macro', 'poster', 'capital']:
            obj = FontBitMap('testing ', font)
            for char in string.letters + string.digits:
            #for char in string.punctuation + string.letters + string.digits:
                char = obj.getChar(char)
                char = obj._shaveChar(char)
                #obj._printChar(char)


#-----------------------------------------------------------------||||||||||||--
if __name__ == '__main__':
    from athenaCL.test import baseTest
    baseTest.main(Test)


















